@@ -1,7 +1,7 @@
+use std::collections::HashMap;
 use std::fs::File;
-use std::io::{Read, Write};
-use std::net::{IpAddr, Ipv4Addr, SocketAddr, SocketAddrV4};
-use std::process::{Command, Stdio};
+use std::io::Read;
+use std::net::{Ipv4Addr, SocketAddr, SocketAddrV4, ToSocketAddrs};
 use std::sync::mpsc::{channel, Receiver, Sender};
 use std::time;
 
@@ -10,6 +10,7 @@ use log::*;
 use yaml_rust::YamlLoader;
 
 use wg_netmanager::configuration::*;
+use wg_netmanager::crypt_udp::AddressedTo;
 use wg_netmanager::crypt_udp::CryptUdp;
 use wg_netmanager::crypt_udp::UdpPacket;
 use wg_netmanager::error::*;
@@ -18,54 +19,6 @@ use wg_netmanager::manager::*;
 use wg_netmanager::tui_display::TuiApp;
 use wg_netmanager::wg_dev::*;
 
-// ===================== Logging Set Up =====================
-fn set_up_logging(log_filter: log::LevelFilter) {
-    use fern::colors::*;
-    // configure colors for the whole line
-    let colors_line = ColoredLevelConfig::new()
-        .error(Color::Red)
-        .warn(Color::Yellow)
-        // we actually don't need to specify the color for debug and info, they are white by default
-        .info(Color::White)
-        .debug(Color::Blue)
-        // depending on the terminals color scheme, this is the same as the background color
-        .trace(Color::BrightBlack);
-
-    // configure colors for the name of the level.
-    // since almost all of them are the same as the color for the whole line, we
-    // just clone `colors_line` and overwrite our changes
-    let colors_level = colors_line.info(Color::Green);
-    // here we set up our fern Dispatch
-    fern::Dispatch::new()
-        .format(move |out, message, record| {
-            out.finish(format_args!(
-                "{color_line}{date} {level} {color_line}{message}\x1B[0m",
-                color_line = format_args!(
-                    "\x1B[{}m",
-                    colors_line.get_color(&record.level()).to_fg_str()
-                ),
-                date = chrono::Local::now().format("%H:%M:%S"),
-                //target = record.target(),
-                level = colors_level.color(record.level()),
-                message = message,
-            ));
-        })
-        // set the default log level. to filter out verbose log messages from dependencies, set
-        // this to Warn and overwrite the log level for your crate.
-        .level(log_filter)
-        // change log levels for individual modules. Note: This looks for the record's target
-        // field which defaults to the module path but can be overwritten with the `target`
-        // parameter:
-        // `info!(target="special_target", "This log message is about special_target");`
-        //.level_for("pretty_colored", log::LevelFilter::Trace)
-        // output to stdout
-        .chain(std::io::stdout())
-        .apply()
-        .unwrap();
-
-    debug!("finished setting up logging! yay!");
-}
-
 fn main() -> BoxResult<()> {
     let matches = App::new("Wireguard Network Manager")
         .version("0.1")
@@ -90,7 +43,7 @@ fn main() -> BoxResult<()> {
                 .short("w")
                 .long("wireguard-port")
                 .value_name("PORT")
-                .default_value("55555")
+                .default_value("50001")
                 .help("Wireguard udp port aka Listen port, if not defined in config file")
                 .takes_value(true),
         )
@@ -99,7 +52,7 @@ fn main() -> BoxResult<()> {
                 .short("u")
                 .long("admin-port")
                 .value_name("PORT")
-                .default_value("50000")
+                .default_value("55551")
                 .help("udp port for encrypted communication, if not defined in config file")
                 .takes_value(true),
         )
@@ -108,6 +61,11 @@ fn main() -> BoxResult<()> {
                 .short("t")
                 .help("Use text user interface"),
         )
+        .arg(
+            Arg::with_name("logfile")
+                .short("l")
+                .help("log to file <name>.log"),
+        )
         .arg(
             Arg::with_name("v")
                 .short("v")
@@ -136,12 +94,21 @@ fn main() -> BoxResult<()> {
 
     let use_tui = matches.is_present("tui");
     let use_existing_interface = matches.is_present("existing_interface");
+    let computer_name = matches.value_of("name").unwrap();
 
     // Select logger based on command line flag
     //
+    let opt_fname = if matches.is_present("logfile") {
+        Some(format!("{}.log", computer_name))
+    } else {
+        None
+    };
     if use_tui {
         tui_logger::init_logger(log::LevelFilter::Trace).unwrap();
         tui_logger::set_default_level(log::LevelFilter::Trace);
+        if let Some(fname) = opt_fname {
+            tui_logger::set_log_file(&fname)?;
+        }
     } else {
         let log_filter = match matches.occurrences_of("v") {
             0 => log::LevelFilter::Error,
@@ -150,14 +117,14 @@ fn main() -> BoxResult<()> {
             3 => log::LevelFilter::Debug,
             _ => log::LevelFilter::Trace,
         };
-        set_up_logging(log_filter);
+        set_up_logging(log_filter, opt_fname)?;
     }
 
     let interface = matches.value_of("interface").unwrap();
     let wg_ip: Ipv4Addr = matches.value_of("wg_ip").unwrap().parse().unwrap();
     let wg_port: u16 = matches.value_of("wireguard_port").unwrap().parse().unwrap();
     let admin_port: u16 = matches.value_of("admin_port").unwrap().parse().unwrap();
-    let computer_name = matches.value_of("name").unwrap();
+
     #[cfg(target_os = "linux")]
     let ip_list = wg_netmanager::interfaces::get();
     #[cfg(not(target_os = "linux"))]
@@ -175,47 +142,39 @@ fn main() -> BoxResult<()> {
 
     let network = &conf[0]["network"];
     let shared_key = base64::decode(&network["sharedKey"].as_str().unwrap()).unwrap();
+    let subnet: ipnet::Ipv4Net = network["subnet"].as_str().unwrap().parse().unwrap();
 
-    let mut peers: Vec<PublicPeer> = vec![];
+    if !subnet.contains(&wg_ip) {
+        return Err(format!("{} is outside of {}", wg_ip, subnet).into());
+    }
+
+    let mut peers: HashMap<Ipv4Addr, PublicPeer> = HashMap::new();
     for p in conf[0]["peers"].as_vec().unwrap() {
         info!("STATIC PEER: {:#?}", p);
-        let public_ip: IpAddr = p["publicIp"].as_str().unwrap().parse().unwrap();
-        let wg_port = p["wgPort"].as_i64().unwrap() as u16;
+        let endpoint = p["endPoint"].as_str().unwrap().to_string();
+        let mut flds = endpoint.split(':').collect::<Vec<_>>();
+        let port_str = flds.pop().unwrap();
+        let wg_port = (*port_str).parse::<u16>().unwrap();
         let admin_port = p["adminPort"].as_i64().unwrap() as u16;
         let wg_ip: Ipv4Addr = p["wgIp"].as_str().unwrap().parse().unwrap();
         let pp = PublicPeer {
-            public_ip,
-            wg_port,
+            endpoint,
             admin_port,
+            wg_port,
             wg_ip,
         };
-        peers.push(pp);
+        peers.insert(wg_ip, pp);
     }
 
-    let output = Command::new("wg")
-        .arg("genkey")
-        .stdout(Stdio::piped())
-        .output()?
-        .stdout;
-    let raw_private_key = String::from_utf8_lossy(&output);
-    let my_private_key = raw_private_key.trim();
+    let wg_dev = get_wireguard_device(interface)?;
+    let (my_private_key, my_public_key) = wg_dev.create_key_pair()?;
     trace!("My private key: {}", my_private_key);
-    let mut cmd = Command::new("wg")
-        .arg("pubkey")
-        .stdin(Stdio::piped())
-        .stdout(Stdio::piped())
-        .spawn()?;
-    write!(cmd.stdin.as_ref().unwrap(), "{}", my_private_key)?;
-    cmd.wait()?;
-    let mut public_key = String::new();
-    cmd.stdout.unwrap().read_to_string(&mut public_key)?;
-    let my_public_key = public_key.trim();
+    trace!("My public key: {}", my_public_key);
     let timestamp = wg_netmanager::util::now();
     let my_public_key_with_time = PublicKeyWithTime {
-        key: my_public_key.to_string(),
+        key: my_public_key,
         priv_key_creation_time: timestamp,
     };
-    trace!("My public key: {}", my_public_key);
 
     let static_config = StaticConfiguration::builder()
         .name(computer_name)
@@ -224,6 +183,7 @@ fn main() -> BoxResult<()> {
         .wg_name(interface)
         .wg_port(wg_port)
         .admin_port(admin_port)
+        .subnet(subnet)
         .shared_key(shared_key)
         .my_public_key(my_public_key_with_time)
         .my_private_key(my_private_key)
@@ -232,10 +192,10 @@ fn main() -> BoxResult<()> {
         .use_existing_interface(use_existing_interface)
         .build();
 
-    run(&static_config)
+    run(&static_config, wg_dev)
 }
 
-fn run(static_config: &StaticConfiguration) -> BoxResult<()> {
+fn run(static_config: &StaticConfiguration, mut wg_dev: Box<dyn WireguardDevice>) -> BoxResult<()> {
     let (tx, rx) = channel();
 
     let tx_handler = tx.clone();
@@ -286,8 +246,6 @@ fn run(static_config: &StaticConfiguration) -> BoxResult<()> {
         }
     });
 
-    let wg_dev = WireguardDeviceLinux::init(&static_config.wg_name);
-
     // in case there are dangling routes
     if !static_config.use_existing_interface {
         wg_dev.take_down_device().ok();
@@ -297,7 +255,7 @@ fn run(static_config: &StaticConfiguration) -> BoxResult<()> {
         wg_dev.flush_all()?;
     }
 
-    wg_dev.set_ip(&static_config.wg_ip)?;
+    wg_dev.set_ip(&static_config.wg_ip, &static_config.subnet)?;
 
     let mut tui_app = if static_config.use_tui {
         TuiApp::init(tx.clone())?
@@ -305,7 +263,7 @@ fn run(static_config: &StaticConfiguration) -> BoxResult<()> {
         TuiApp::off()
     };
 
-    let rc = main_loop(static_config, &wg_dev, crypt_socket, tx, rx, &mut tui_app);
+    let rc = main_loop(static_config, &*wg_dev, crypt_socket, tx, rx, &mut tui_app);
 
     if !static_config.use_existing_interface {
         wg_dev.take_down_device().ok();
@@ -319,7 +277,7 @@ fn run(static_config: &StaticConfiguration) -> BoxResult<()> {
 fn main_loop(
     static_config: &StaticConfiguration,
     wg_dev: &dyn WireguardDevice,
-    crypt_socket: CryptUdp,
+    mut crypt_socket: CryptUdp,
     tx: Sender<Event>,
     rx: Receiver<Event>,
     tui_app: &mut TuiApp,
@@ -327,14 +285,16 @@ fn main_loop(
     let mut network_manager = NetworkManager::new(static_config.wg_ip);
 
     // set up initial wireguard configuration without peers
-    tx.send(Event::PeerListChange).unwrap();
+    tx.send(Event::UpdateWireguardConfiguration).unwrap();
     tx.send(Event::SendAdvertisementToPublicPeers).unwrap();
 
+    //let mut timed_events: Vec<Vec<Event>> = vec![];
+
     let mut tick_cnt = 0;
     loop {
-        trace!("Main loop: {} peers", network_manager.peer.len());
+        trace!(target: "loop", "Main loop");
         let evt = rx.recv();
-        trace!("{:?}", evt);
+        trace!(target: "loop", "{:?}", evt);
         match evt {
             Err(e) => {
                 error!("Receive error: {:?}", e);
@@ -356,12 +316,25 @@ fn main_loop(
                 }
                 if tick_cnt % 30 == 2 {
                     // every 30s
-                    info!("Main loop: {} peers", network_manager.peer.len());
+                    network_manager.stats();
                 }
                 if tick_cnt % 60 == 3 {
                     // every 60s
                     tx.send(Event::SendAdvertisementToPublicPeers).unwrap();
                 }
+
+                //if !timed_events.is_empty() {
+                //    let events = timed_events.remove(0);
+                //    for evt in events.into_iter() {
+                //        tx.send(evt).unwrap();
+                //    }
+                //}
+
+                let events = network_manager.process_new_nodes_every_second(&static_config);
+                for evt in events.into_iter() {
+                    tx.send(evt).unwrap();
+                }
+
                 tick_cnt += 1;
             }
             Ok(Event::SendPingToAllDynamicPeers) => {
@@ -370,21 +343,34 @@ fn main_loop(
                 let ping_peers = network_manager.check_ping_timeouts(10); // should be < half of dead peer timeout
                 for (wg_ip, admin_port) in ping_peers {
                     let destination = SocketAddr::V4(SocketAddrV4::new(wg_ip, admin_port));
-                    tx.send(Event::SendAdvertisement { to: destination })
-                        .unwrap();
+                    tx.send(Event::SendAdvertisement {
+                        addressed_to: AddressedTo::WireguardAddress,
+                        to: destination,
+                        wg_ip,
+                    })
+                    .unwrap();
                 }
             }
             Ok(Event::SendAdvertisementToPublicPeers) => {
                 // These advertisements are sent to the known internet address as defined in the config file.
                 // As all udp packets are encrypted, this should not be an issue.
                 //
-                for peer in static_config.peers.iter() {
+                for peer in static_config.peers.values() {
                     if !network_manager.knows_peer(&peer.wg_ip) {
                         // ensure not to send to myself
                         if peer.wg_ip != static_config.wg_ip {
-                            let destination = SocketAddr::new(peer.public_ip, peer.admin_port);
-                            tx.send(Event::SendAdvertisement { to: destination })
+                            // Resolve here to make it work for dyndns hosts
+                            let endpoints = peer.endpoint.to_socket_addrs()?;
+                            trace!("ENDPOINTS: {:#?}", endpoints);
+                            for sa in endpoints {
+                                let destination = SocketAddr::new(sa.ip(), peer.admin_port);
+                                tx.send(Event::SendAdvertisement {
+                                    addressed_to: AddressedTo::StaticAddress,
+                                    to: destination,
+                                    wg_ip: peer.wg_ip,
+                                })
                                 .unwrap();
+                            }
                         }
                     }
                 }
@@ -394,11 +380,13 @@ fn main_loop(
                 let events: Vec<Event>;
                 match udp_packet {
                     Advertisement(ad) => {
+                        debug!(target: &ad.wg_ip.to_string(), "Received advertisement from {:?}", src_addr);
                         events = network_manager.analyze_advertisement(ad, src_addr);
                     }
                     RouteDatabaseRequest => match src_addr {
                         SocketAddr::V4(destination) => {
                             info!(target: "routing", "RouteDatabaseRequest from {:?}", src_addr);
+                            debug!(target: &destination.ip().to_string(), "Received database request");
                             events = vec![Event::SendRouteDatabase { to: destination }];
                         }
                         SocketAddr::V6(..) => {
@@ -406,37 +394,85 @@ fn main_loop(
                             events = vec![];
                         }
                     },
-                    RouteDatabase(req) => {
+                    RouteDatabase(db) => {
                         info!(target: "routing", "RouteDatabase from {}", src_addr);
-                        events = network_manager.process_route_database(req);
+                        debug!(target: &src_addr.ip().to_string(), "Received route database, version = {}", db.routedb_version);
+                        events = network_manager.process_route_database(db);
+                    }
+                    LocalContactRequest => match src_addr {
+                        SocketAddr::V4(destination) => {
+                            info!(target: "probing", "LocalContactRequest from {:?}", src_addr);
+                            debug!(target: &destination.ip().to_string(), "Received local contact request");
+                            events = vec![Event::SendLocalContact { to: destination }];
+                        }
+                        SocketAddr::V6(..) => {
+                            error!(target: "probing", "Expected IPV4 and not IPV6 address");
+                            events = vec![];
+                        }
+                    },
+                    LocalContact(contact) => {
+                        debug!(target: "probing", "Received contact info: {:#?}", contact);
+                        debug!(target: &contact.wg_ip.to_string(), "Received local contacts");
+                        network_manager.process_local_contact(contact);
+                        events = vec![];
                     }
                 }
                 for evt in events {
                     tx.send(evt).unwrap();
                 }
             }
-            Ok(Event::SendAdvertisement { to: destination }) => {
+            Ok(Event::SendAdvertisement {
+                addressed_to,
+                to: destination,
+                wg_ip,
+            }) => {
+                debug!(target: &wg_ip.to_string(),"Send advertisement to {:?}", destination);
                 let routedb_version = network_manager.db_version();
-                let advertisement =
-                    UdpPacket::advertisement_from_config(static_config, routedb_version);
+                let opt_dp: Option<&DynamicPeer> = network_manager.dynamic_peer_for(&wg_ip);
+                let advertisement = UdpPacket::advertisement_from_config(
+                    static_config,
+                    routedb_version,
+                    addressed_to,
+                    opt_dp,
+                );
                 let buf = serde_json::to_vec(&advertisement).unwrap();
                 info!(target: "advertisement", "Send advertisement to {}", destination);
                 crypt_socket.send_to(&buf, destination).ok();
             }
             Ok(Event::SendRouteDatabaseRequest { to: destination }) => {
+                debug!(target: &destination.ip().to_string(), "Send route database request to {:?}", destination);
                 let request = UdpPacket::route_database_request();
                 let buf = serde_json::to_vec(&request).unwrap();
                 info!(target: "routing", "Send RouteDatabaseRequest to {}", destination);
-                crypt_socket.send_to(&buf, destination).ok();
+                crypt_socket.send_to(&buf, SocketAddr::V4(destination)).ok();
             }
             Ok(Event::SendRouteDatabase { to: destination }) => {
+                debug!(target: &destination.ip().to_string(), "Send route database to {:?}", destination);
                 let packages = network_manager.provide_route_database();
                 for p in packages {
                     let buf = serde_json::to_vec(&p).unwrap();
                     info!(target: "routing", "Send RouteDatabase to {}", destination);
-                    crypt_socket.send_to(&buf, destination).ok();
+                    crypt_socket.send_to(&buf, SocketAddr::V4(destination)).ok();
                 }
             }
+            Ok(Event::SendLocalContactRequest { to: destination }) => {
+                debug!(target: &destination.ip().to_string(), "Send local contact request to {:?}", destination);
+                let request = UdpPacket::local_contact_request();
+                let buf = serde_json::to_vec(&request).unwrap();
+                info!(target: "probing", "Send LocalContactRequest to {}", destination);
+                crypt_socket.send_to(&buf, SocketAddr::V4(destination)).ok();
+            }
+            Ok(Event::SendLocalContact { to: destination }) => {
+                debug!(target: &destination.ip().to_string(), "Send local contacts to {:?}", destination);
+                let local_contact = UdpPacket::local_contact_from_config(
+                    static_config,
+                    network_manager.my_visible_wg_endpoint,
+                );
+                trace!(target: "probing", "local contact to {:#?}", local_contact);
+                let buf = serde_json::to_vec(&local_contact).unwrap();
+                info!(target: "probing", "Send local contact to {}", destination);
+                crypt_socket.send_to(&buf, SocketAddr::V4(destination)).ok();
+            }
             Ok(Event::CheckAndRemoveDeadDynamicPeers) => {
                 network_manager.output();
                 let dead_peers = network_manager.check_timeouts(120);
@@ -445,46 +481,51 @@ fn main_loop(
                 }
                 if !dead_peers.is_empty() {
                     for wg_ip in dead_peers {
+                        debug!(target: &wg_ip.to_string(), "is dead => remove");
                         debug!(target: "dead_peer", "Found dead peer {}", wg_ip);
                         network_manager.remove_dynamic_peer(&wg_ip);
                     }
-                    tx.send(Event::PeerListChange).unwrap();
+                    tx.send(Event::UpdateWireguardConfiguration).unwrap();
                     tx.send(Event::UpdateRoutes).unwrap();
                 }
             }
-            Ok(Event::PeerListChange) => {
+            Ok(Event::UpdateWireguardConfiguration) => {
                 info!("Update peers");
                 let conf = static_config.as_conf_as_peer(&network_manager);
-                info!("Configuration as peer\n{}\n", conf);
+                info!(target: "wireguard", "Configuration as peer\n{}\n", conf);
                 wg_dev.sync_conf(&conf)?;
             }
+            Ok(Event::ReadWireguardConfiguration) => {
+                let pubkey_to_endpoint = wg_dev.retrieve_conf()?;
+                network_manager.current_wireguard_configuration(pubkey_to_endpoint);
+            }
             Ok(Event::UpdateRoutes) => {
                 let changes = network_manager.get_route_changes();
                 for rc in changes {
                     use RouteChange::*;
                     debug!("{:?}", rc);
                     match rc {
-                        AddRoute {
-                            to,
-                            gateway: Some(gateway),
-                        } => {
-                            wg_dev.add_route(&format!("{}/32", to), Some(gateway))?;
+                        AddRoute { to, gateway } => {
+                            debug!(target: &to.to_string(), "add route with gateway {:?}", gateway);
+                            wg_dev.add_route(&format!("{}/32", to), gateway)?;
                         }
-                        AddRoute { to, gateway: None } => {
-                            wg_dev.add_route(&format!("{}/32", to), None)?;
+                        ReplaceRoute { to, gateway } => {
+                            debug!(target: &to.to_string(), "replace route with gateway {:?}", gateway);
+                            wg_dev.replace_route(&format!("{}/32", to), gateway)?;
                         }
-                        DelRoute {
-                            to,
-                            gateway: Some(gateway),
-                        } => {
-                            wg_dev.del_route(&format!("{}/32", to), Some(gateway))?;
-                        }
-                        DelRoute { to, gateway: None } => {
-                            wg_dev.del_route(&format!("{}/32", to), None)?;
+                        DelRoute { to, gateway } => {
+                            debug!(target: &to.to_string(), "del route with gateway {:?}", gateway);
+                            wg_dev.del_route(&format!("{}/32", to), gateway)?;
                         }
                     }
                 }
-                tx.send(Event::PeerListChange).unwrap();
+                tx.send(Event::UpdateWireguardConfiguration).unwrap();
+
+                // all routes have been updated. So process new nodes (not ideal solution here)
+                let events = network_manager.process_new_nodes_every_second(static_config);
+                for evt in events {
+                    tx.send(evt).unwrap();
+                }
             }
             Ok(Event::TuiApp(evt)) => {
                 tui_app.process_event(evt);
